package hu.bme.mit.bookorder;

import hu.bme.mit.bookorder.book.Book;
import hu.bme.mit.bookorder.book.Genre;

import org.drools.KnowledgeBase;
import org.drools.builder.DecisionTableConfiguration;
import org.drools.builder.DecisionTableInputType;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.logger.KnowledgeRuntimeLogger;
import org.drools.logger.KnowledgeRuntimeLoggerFactory;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.rule.QueryResults;
import org.drools.runtime.rule.QueryResultsRow;

public class BookBidder {
	
	KnowledgeBase kBase = readKnowledgeBase();
	StatefulKnowledgeSession kSession = kBase.newStatefulKnowledgeSession();
	KnowledgeRuntimeLogger logger; 
	
	public void bidForBooks(Iterable<Object> objects) {
		logger = KnowledgeRuntimeLoggerFactory.newThreadedFileLogger(kSession, "logs/auditLog", 200);

		for (Object object : objects) {
			/*FactHandle handle = */ kSession.insert(object); // !!			
		}
		fireAgendaGroup("init-price");	
		fireAgendaGroup("adjust-price");
		fireAgendaGroup("bestsellers");	
		fireAgendaGroup("bestsellers-adjust");	
		fireAgendaGroup("volume-price");
		printOutput();
		
		logger.close();
	}


	private void printOutput() {
		for (Genre genre : Genre.values()) {
			QueryResults results =
				kSession.getQueryResults( "booksOfGenre", new Object[] { genre } );
			int numResults = results.size();
			if (numResults == 0) {
				System.out.println( String.format("*** No bids for books of genre %s", genre) );
			} else {
				System.out.println( String.format(
						"*** Genre %s (%d bid%s)", 
						genre, numResults, numResults > 1 ? "s" : "") );
				for ( QueryResultsRow row : results ) {
					Book book = (Book) row.get("book");
					System.out.println(String.format(
							"\t Offering %s", 
							book));		
				}

			}
		}
	}

	private void fireAgendaGroup(String agendaGroup) {
		kSession.getAgenda().getAgendaGroup(agendaGroup).setFocus();
		kSession.fireAllRules();
	}

	private static KnowledgeBase readKnowledgeBase() {
		KnowledgeBuilder kBuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
        DecisionTableConfiguration dtConfig = KnowledgeBuilderFactory.newDecisionTableConfiguration();
        dtConfig.setInputType(DecisionTableInputType.XLS);
        
        kBuilder.add(ResourceFactory.newClassPathResource("init.drl"), ResourceType.DRL);	// !!
        kBuilder.add(ResourceFactory.newClassPathResource("adjust.drl"), ResourceType.DRL); // !!
        kBuilder.add(ResourceFactory.newClassPathResource("bestsellers.drl"), ResourceType.DRL); // !!
        kBuilder.add(ResourceFactory.newClassPathResource("query.drl"), ResourceType.DRL);	// !!
        kBuilder.add(ResourceFactory.newClassPathResource("volume.xls"), ResourceType.DTABLE, dtConfig);
        
		return kBuilder.newKnowledgeBase();
	}
}
